@@ -278,8 +278,8 @@ class mainwindow(ttk.Frame):
         self.bind_all("<Return>", self.distset_entry)
         self.bind_all("<Shift-Left>", self.press_arrowkey)
         self.bind_all("<Shift-Right>", self.press_arrowkey)
-    def open_mapfile(self, event=None):
-        inputdir = filedialog.askopenfilename()
+    def open_mapfile(self, event=None,inputdir=None):
+        inputdir = filedialog.askopenfilename() if inputdir == None else inputdir
         if inputdir != '':
             self.filedir_entry_val.set(inputdir)
             
@@ -512,10 +512,7 @@ class mainwindow(ttk.Frame):
         filepath = filedialog.asksaveasfilename(filetypes=[('portable network graphics (png)','*.png'), ('scalable vector graphics (svg)','*.svg'), ('any format','*')], defaultextension='*.*')
         if filepath != '':
             filepath = pathlib.Path(filepath)
-            #self.fig_plane.savefig(filepath.with_stem(filepath.stem + '_plane'))
-            #self.fig_profile.savefig(filepath.with_stem(filepath.stem + '_profile'))
-            self.fig_plane.savefig(filepath.parent.joinpath(str(filepath.stem) + '_plane' + str(filepath.suffix)))
-            self.fig_profile.savefig(filepath.parent.joinpath(str(filepath.stem) + '_profile' + str(filepath.suffix)))
+            self.fig_plane.savefig(filepath.parent.joinpath(str(filepath.stem) + '' + str(filepath.suffix)))
     def save_trackdata(self, event=None):
         filepath = filedialog.askdirectory(initialdir='./')
         if filepath != '':
@@ -524,13 +521,13 @@ class mainwindow(ttk.Frame):
             
             output_filename = filepath.joinpath(str(filename_base)+'_owntrack'+'.csv')
             output = self.result.owntrack_pos
-            header = 'distance,x,y,z,direction,radius,gradient'
+            header = 'distance,x,y,z,direction,radius,gradient,interpolate_func,cant,center,gauge'
             np.savetxt(output_filename, output, delimiter=',',header=header,fmt='%.6f')
             
             for key in self.result.othertrack_pos.keys():
                 output_filename = filepath.joinpath(str(filename_base)+'_'+key+'.csv')
                 output = self.result.othertrack_pos[key]
-                header = 'distance,x,y,z'
+                header = 'distance,x,y,z,interpolate_func,cant,center,gauge'
                 np.savetxt(output_filename, output, delimiter=',',header=header,fmt='%.6f')
     def set_plotlimit(self, event=None):
         if self.result != None:
@@ -615,11 +612,14 @@ def main():
                 traceback.print_exception(type, value, tb)
                 # ...then start the debugger in post-mortem mode
                 pdb.pm()
-        import sys
+        #import sys
         sys.excepthook = info
         print('Debug mode')
     
     tk.CallWrapper = Catcher
     root = tk.Tk()
     app = mainwindow(master=root, parser = None)
+
+    if len(sys.argv)>1:
+        app.open_mapfile(inputdir=sys.argv[1])
     app.mainloop()
