@@ -1,18 +1,22 @@
+#
+#    Copyright 2021 konawasabi
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
+#
+
 '''
-    Copyright 2021 konawasabi
-
-    Licensed under the Apache License, Version 2.0 (the "License");
-    you may not use this file except in compliance with the License.
-    You may obtain a copy of the License at
-
-        http://www.apache.org/licenses/LICENSE-2.0
-
-    Unless required by applicable law or agreed to in writing, software
-    distributed under the License is distributed on an "AS IS" BASIS,
-    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-    See the License for the specific language governing permissions and
-    limitations under the License.
 '''
+
 import numpy as np
 from . import trackcoordinate as tc
 
@@ -70,35 +74,59 @@ class TrackGenerator():
         self.last_pos['gradient']        = gr0    if gr0    != None else 0
         self.last_pos['distance']        = dist0  if dist0  != None else min(self.list_cp)
         self.last_pos['interpolate_func'] = 'line'
+        self.last_pos['cant']            = 0
+        self.last_pos['center']          = 0
+        self.last_pos['gauge']           = 0
         
         self.radius_lastpos = {}
         self.radius_lastpos['distance'] = self.last_pos['distance']
         self.radius_lastpos['theta']    = self.last_pos['theta']
-        self.radius_lastpos['radius']  = self.last_pos['radius']
+        self.radius_lastpos['radius']   = self.last_pos['radius']
         
         #座標情報を格納するリスト
-        self.result = [[self.last_pos['distance'],self.last_pos['x'],self.last_pos['y'],self.last_pos['z'],self.last_pos['theta'],self.last_pos['radius'],self.last_pos['gradient']]]
+        self.result = []
+        '''
+        self.result = [[self.last_pos['distance'],\
+                        self.last_pos['x'],\
+                        self.last_pos['y'],\
+                        self.last_pos['z'],\
+                        self.last_pos['theta'],\
+                        self.last_pos['radius'],\
+                        self.last_pos['gradient'],\
+                        1,\
+                        self.last_pos['cant'],\
+                        self.last_pos['center'],\
+                        self.last_pos['gauge']]]
+        '''
         
+        #縦断面図曲線半径情報を格納するリスト
         self.radius_dist = []
     def generate_owntrack(self):
         '''マップ要素が存在する全ての距離程(self.list_cp)に対して自軌道の座標データを生成する。
         self.env: マップ要素が格納されたEnvironmentオブジェクト。
-        結果はself.result に[[distance,xpos,ypos,zpos,theta,radius,gradient],[d.,x.,y.,...],[...],...]として格納する。
+        結果はself.result に[[distance,xpos,ypos,zpos,theta,radius,gradient,interpolate_func,cant,center,gauge],[d.,x.,y.,...],[...],...]として格納する。
         '''
         radius_p      = TrackPointer(self.env,'radius')
         gradient_p    = TrackPointer(self.env,'gradient')
         turn_p        = TrackPointer(self.env,'turn')
         interpolate_p = TrackPointer(self.env,'interpolate_func')
+        cant_p        = TrackPointer(self.env,'cant')
+        center_p      = TrackPointer(self.env,'center')
+        gauge_p       = TrackPointer(self.env,'gauge')
         
-        grad_gen = tc.gradient_intermediate()
+        grad_gen  = tc.gradient_intermediate()
         curve_gen = tc.curve_intermediate()
+        cant_gen  = tc.Cant(cant_p, self.data_ownt, self.last_pos)
         
         if not __debug__: # -O オプションが指定されている時のみ、デバッグ情報を処理
             # numpy RuntimeWarning発生時に当該点の距離程を印字
-            def print_warning_position(err,flag):
+            def raise_warning_position(err,flag):
                 raise RuntimeWarning('Numpy warning: '+str(err)+', '+str(flag)+' at '+str(dist))
+            def print_warning_position(err,flag):
+                print('Numpy warning: '+str(err)+', '+str(flag)+' at '+str(dist))
             np.seterr(all='call')
             np.seterrcall(print_warning_position)
+            #np.seterrcall(raise_warning_position)
             
         #import pdb
         #pdb.set_trace()
@@ -106,21 +134,31 @@ class TrackGenerator():
         for dist in self.list_cp:
             # curve.setfunction に対する処理
             while (interpolate_p.onNextpoint(dist)): #注目している要素区間の終端に到達？
-                #if(interpolate_p.pointer['next'] != None):
                 self.last_pos['interpolate_func'] = self.data_ownt[interpolate_p.pointer['next']]['value']
-                #print(self.data_ownt[interpolate_p.pointer['next']]['distance'],self.last_pos['interpolate_func'])
                 interpolate_p.seeknext()
+                
+            # curve.setcenter に対する処理
+            center_tmp = self.last_pos['center']
+            while (center_p.onNextpoint(dist)): #注目している要素区間の終端に到達？
+                center_tmp = self.data_ownt[center_p.pointer['next']]['value']
+                center_p.seeknext()
+                
+            # curve.setgauge に対する処理
+            gauge_tmp = self.last_pos['gauge']
+            while (gauge_p.onNextpoint(dist)): #注目している要素区間の終端に到達？
+                gauge_tmp = self.data_ownt[gauge_p.pointer['next']]['value']
+                gauge_p.seeknext()
             
             # radiusに対する処理
             while (radius_p.overNextpoint(dist)): #注目している要素区間の終端を超えたか？
                 if(radius_p.seekoriginofcontinuous(radius_p.pointer['next']) != None):
-                    self.last_pos['radius'] = self.data_ownt[radius_p.seekoriginofcontinuous(radius_p.pointer['next'])]['value']
-                    self.radius_lastpos['radius']  = self.data_ownt[radius_p.seekoriginofcontinuous(radius_p.pointer['next'])]['value']
+                    self.last_pos['radius']         = self.data_ownt[radius_p.seekoriginofcontinuous(radius_p.pointer['next'])]['value']
+                    self.radius_lastpos['radius']   = self.data_ownt[radius_p.seekoriginofcontinuous(radius_p.pointer['next'])]['value']
                     self.radius_lastpos['distance'] = self.data_ownt[radius_p.seekoriginofcontinuous(radius_p.pointer['next'])]['distance']
                     self.radius_lastpos['theta']    = self.last_pos['theta']
                 radius_p.seeknext()
-            if(radius_p.pointer['last'] == None): # 最初のcurve要素に到達していない場合
-                if(radius_p.pointer['next'] == None): # curve要素が存在しないマップの場合
+            if(radius_p.pointer['last'] is None): # 最初のcurve要素に到達していない場合
+                if(radius_p.pointer['next'] is None): # curve要素が存在しないマップの場合
                     if(self.last_pos['radius'] == 0):
                         [x, y] =curve_gen.straight(self.cp_max - self.cp_min,\
                                                     self.last_pos['theta'],\
@@ -143,7 +181,7 @@ class TrackGenerator():
                                                             self.last_pos['theta'],\
                                                             dist - self.last_pos['distance'])
                     radius = self.last_pos['radius']
-            elif(radius_p.pointer['next'] == None): # curve要素リスト終端に到達
+            elif(radius_p.pointer['next'] is None): # curve要素リスト終端に到達
                 if(self.last_pos['radius'] == 0): # 曲線半径が0 (直線)の場合
                     [x, y] = curve_gen.straight(self.cp_max - self.last_pos['distance'],\
                                               self.last_pos['theta'],\
@@ -231,8 +269,8 @@ class TrackGenerator():
                     self.last_pos['gradient']  = self.data_ownt[gradient_p.seekoriginofcontinuous(gradient_p.pointer['next'])]['value']
                     self.last_pos['dist_grad'] = self.data_ownt[gradient_p.seekoriginofcontinuous(gradient_p.pointer['next'])]['distance']
                 gradient_p.seeknext()
-            if(gradient_p.pointer['last'] == None): #最初の勾配要素に到達していない
-                if(gradient_p.pointer['next'] == None): # 勾配が存在しないmapの場合の処理
+            if(gradient_p.pointer['last'] is None): #最初の勾配要素に到達していない
+                if(gradient_p.pointer['next'] is None): # 勾配が存在しないmapの場合の処理
                     z = grad_gen.straight(self.cp_max - self.cp_min,\
                                             self.last_pos['gradient'],\
                                             dist - self.last_pos['distance'])
@@ -241,7 +279,7 @@ class TrackGenerator():
                                             self.last_pos['gradient'],\
                                             dist - self.last_pos['distance'])
                 gradient = self.last_pos['gradient']
-            elif(gradient_p.pointer['next'] == None): #最後の勾配要素を通過した
+            elif(gradient_p.pointer['next'] is None): #最後の勾配要素を通過した
                 z = grad_gen.straight(self.cp_max - self.last_pos['distance'],\
                                         self.last_pos['gradient'],\
                                         dist - self.last_pos['distance'])
@@ -269,6 +307,9 @@ class TrackGenerator():
                                                 self.last_pos['gradient'],\
                                                 dist - self.last_pos['distance'])
                         gradient = self.last_pos['gradient']
+                        
+            #Cantに対する処理
+            cant_tmp = cant_gen.process(dist, self.last_pos['interpolate_func'])
             
             # 地点情報を更新
             self.last_pos['x']       += x
@@ -278,14 +319,48 @@ class TrackGenerator():
             self.last_pos['radius']   = radius
             self.last_pos['gradient'] = gradient
             self.last_pos['distance'] = dist
+            #self.last_pos['interpolate_func'] = 'line'
+            self.last_pos['cant']            = cant_tmp
+            self.last_pos['center']          = center_tmp
+            self.last_pos['gauge']           = gauge_tmp
             # 座標リストに追加
+            '''
+            if self.result is None:
+                self.result = [dist,\
+                    self.last_pos['x'],\
+                    self.last_pos['y'],\
+                    self.last_pos['z'],\
+                    self.last_pos['theta'],\
+                    self.last_pos['radius'],\
+                    self.last_pos['gradient'],\
+                    0 if self.last_pos['interpolate_func'] == 'sin' else 1,\
+                    self.last_pos['cant'],\
+                    self.last_pos['center'],\
+                    self.last_pos['gauge']]
+            else:
+                self.result.append([dist,\
+                    self.last_pos['x'],\
+                    self.last_pos['y'],\
+                    self.last_pos['z'],\
+                    self.last_pos['theta'],\
+                    self.last_pos['radius'],\
+                    self.last_pos['gradient'],\
+                    0 if self.last_pos['interpolate_func'] == 'sin' else 1,\
+                    self.last_pos['cant'],\
+                    self.last_pos['center'],\
+                    self.last_pos['gauge']])
+            '''
             self.result.append([dist,\
                 self.last_pos['x'],\
                 self.last_pos['y'],\
                 self.last_pos['z'],\
                 self.last_pos['theta'],\
                 self.last_pos['radius'],\
-                self.last_pos['gradient']])
+                self.last_pos['gradient'],\
+                0 if self.last_pos['interpolate_func'] == 'sin' else 1,\
+                self.last_pos['cant'],\
+                self.last_pos['center'],\
+                self.last_pos['gauge']])
             
         return np.array(self.result)
     def generate_curveradius_dist(self):
@@ -347,7 +422,7 @@ class TrackPointer():
         self.pointer['next'] = self.seek(0)
     def seeknext(self):
         '''次の要素が存在するインデックスを探し、self.pointer['last', 'next']を書き換える。
-        self.pointer['next'] == None の場合は何もしない。
+        self.pointer['next'] is None の場合は何もしない。
         '''
         if(self.pointer['next'] != None):
             self.pointer['last'] = self.pointer['next']
@@ -360,19 +435,19 @@ class TrackPointer():
     def onNextpoint(self,distance):
         '''注目している要素区間の終端にいるか調べる。
         与えられたdistance == 注目しているpointer['next'] ならTrue。
-        pointer['next'] == None (要素リスト終端に到達した) なら必ずFalse。
+        pointer['next'] is None (要素リスト終端に到達した) なら必ずFalse。
         '''
         return (self.data[self.pointer['next']]['distance'] == distance) if self.pointer['next'] != None else False
     def overNextpoint(self,distance):
         '''注目している要素区間を超えたか調べる。
         与えられたdistance > 注目しているpointer['next'] ならTrue。
-        pointer['next'] == None (要素リスト終端に到達した) なら必ずFalse。
+        pointer['next'] is None (要素リスト終端に到達した) なら必ずFalse。
         '''
         return (self.data[self.pointer['next']]['distance'] < distance) if self.pointer['next'] != None else False
     def beforeLastpoint(self,distance):
         '''注目している要素区間にまだ到達していないか調べる。
         与えられたdistance <= 注目しているpointer['last'] ならTrue。
-        pointer['last'] == None (リスト始端の要素地点に到達していない) なら必ずTrue。
+        pointer['last'] is None (リスト始端の要素地点に到達していない) なら必ずTrue。
         '''
         return (self.data[self.pointer['last']]['distance'] >= distance) if self.pointer['last'] != None else True
     def seekoriginofcontinuous(self,index):
@@ -407,7 +482,7 @@ class OtherTrackGenerator():
         self.distrange={'min':min(self.data, key=lambda x: x['distance'])['distance'], 'max':max(self.data, key=lambda x: x['distance'])['distance']}
         # 前回処理した地点の情報
         self.pos = {'last':{}, 'next':{}}
-        for key in ['x.position','x.radius','x.distance','y.position','y.radius','y.distance']:
+        for key in ['x.position','x.radius','x.distance','y.position','y.radius','y.distance','interpolate_func','cant','center','gauge']:
             self.pos['last'][key] = 0
             self.pos['next'][key] = 0
     
@@ -415,24 +490,26 @@ class OtherTrackGenerator():
         '''他軌道座標を計算する。
         対象はインスタンス作成時に指定したkeyの軌道。
         '''
-        track_gen = tc.OtherTrack() # 座標計算オブジェクト
         # 軌道要素ポインタの作成
         trackptr = {}
-        trackptr['x.position'] = self.OtherTrackPointer(self.env,'x.position',self.trackkey)
-        trackptr['x.radius']   = self.OtherTrackPointer(self.env,'x.radius',self.trackkey)
-        trackptr['y.position'] = self.OtherTrackPointer(self.env,'y.position',self.trackkey)
-        trackptr['y.radius']   = self.OtherTrackPointer(self.env,'y.radius',self.trackkey)
+        for tpkey in ['x.position','x.radius','y.position','y.radius','interpolate_func','cant','center','gauge']:
+            trackptr[tpkey] = self.OtherTrackPointer(self.env,tpkey,self.trackkey)
+            
+        track_gen = tc.OtherTrack() # 座標計算オブジェクト
+        cant_gen  = tc.Cant(trackptr['cant'], self.data, self.pos['last']) # カント計算オブジェクト
+        
         #tp_keys = ['x.position','x.radius','y.position','y.radius']
         #skip_dimension = {'x.position':False, 'x.radius':False, 'y.position':False, 'y.radius':False}
-        for tpkey in trackptr.keys():
+        for tpkey in trackptr.keys(): # ポインタ初期値設定
             if trackptr[tpkey].pointer['next'] != None:
                 for k in ['last','next']:
                     newval = self.data[trackptr[tpkey].pointer['next']]['value']
                     self.pos[k][tpkey] = newval if newval != 'c' else 0
+                    
         for element in self.owntrack_position: # 自軌道が指定されている全ての距離程について計算する
             if self.distrange['min'] > element[0]: # 対象となる軌道が最初に現れる距離程にまだ達していないか？
                 continue
-            for tpkey in trackptr.keys(): # ポインタを進める
+            for tpkey in ['x.position','x.radius','y.position','y.radius']: # ポインタを進める
                 while trackptr[tpkey].overNextpoint(element[0]):
                     trackptr[tpkey].seeknext()
                     if trackptr[tpkey].pointer['next'] != None:
@@ -440,6 +517,15 @@ class OtherTrackGenerator():
                         k = 'next'
                         newval = self.data[trackptr[tpkey].pointer[k]]['value']
                         self.pos[k][tpkey] = newval if newval != 'c' else self.pos['last'][tpkey]
+            for tpkey in ['interpolate_func','center','gauge']: # ポインタを進める
+                while trackptr[tpkey].onNextpoint(element[0]):
+                    trackptr[tpkey].seeknext()
+                    if trackptr[tpkey].pointer['next'] != None:
+                        self.pos['last'][tpkey] = self.pos['next'][tpkey]
+                        k = 'next'
+                        newval = self.data[trackptr[tpkey].pointer[k]]['value']
+                        self.pos[k][tpkey] = newval if newval != 'c' else self.pos['last'][tpkey]
+            
             if trackptr['x.position'].pointer['last'] != None and trackptr['x.position'].pointer['next'] != None: # skip_dimension に従って計算するかどうか判断する
                 for k in ['last','next']:
                     self.pos[k]['x.distance'] = self.data[trackptr['x.position'].pointer[k]]['distance']
@@ -463,5 +549,15 @@ class OtherTrackGenerator():
                                                                 element)
             else:
                 temp_result_Y = [0,self.pos['last']['y.position']]+ np.array([element[1],element[3]])
-            self.result.append([element[0],temp_result_X[0],temp_result_X[1],temp_result_Y[1]])
+                
+            temp_result_cant = cant_gen.process(element[0], self.pos['last']['interpolate_func'])
+            
+            self.result.append([element[0],\
+                                temp_result_X[0],\
+                                temp_result_X[1],\
+                                temp_result_Y[1],\
+                                0 if self.pos['last']['interpolate_func'] == 'sin' else 1,\
+                                temp_result_cant,\
+                                self.pos['last']['center'],\
+                                self.pos['last']['gauge']])
         return np.array(self.result)
